@@ -1,6 +1,6 @@
 /* Machine-dependent pthreads configuration and inline functions.
-   64 bit S/390 version.
-   Copyright (C) 2001, 2002 Free Software Foundation, Inc.
+   S390 version.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
    Contributed by Martin Schwidefsky (schwidefsky@de.ibm.com).
    This file is part of the GNU C Library.
 
@@ -23,7 +23,7 @@
 #define _PT_MACHINE_H   1
 
 #ifndef PT_EI
-# define PT_EI extern inline
+# define PT_EI extern inline __attribute__ ((always_inline))
 #endif
 
 extern long int testandset (int *spinlock);
@@ -66,22 +66,16 @@ register char * stack_pointer __asm__ ("15");
 #define INIT_THREAD_SELF(descr, nr) __builtin_set_thread_pointer (descr)
 #else
 /* Return the thread descriptor for the current thread.
-   64 bit S/390 uses access register 0 and 1 as "thread register".  */
+   S/390 registers uses access register 0 as "thread register".  */
 #define THREAD_SELF  ({                                                       \
   register pthread_descr __self;                                              \
-  __asm__ ("   ear  %0,%%a0\n"                                                \
-           "   sllg %0,%0,32\n"                                               \
-           "   ear  %0,%%a1\n"                                                \
-           : "=d" (__self) );                                                 \
+  __asm__ ("ear %0,%%a0" : "=d" (__self) );                                   \
   __self;                                                                     \
 })
 
 /* Initialize the thread-unique value.  */
 #define INIT_THREAD_SELF(descr, nr)  ({                                       \
-  __asm__ ("   sar  %%a1,%0\n"                                                \
-           "   srlg 0,%0,32\n"                                                \
-           "   sar  %%a0,0\n"                                                 \
-           : : "d" (descr) : "0" );                                           \
+  __asm__ ("sar %%a0,%0" : : "d" (descr) );                                   \
 })
 #endif
 
@@ -111,14 +105,15 @@ __compare_and_swap(long int *p, long int oldval, long int newval)
         int retval;
 
         __asm__ __volatile__(
-                "  lgr  0,%2\n"
-                "  csg  0,%3,%1\n"
+                "  la   1,%1\n"
+                "  lr   0,%2\n"
+                "  cs   0,%3,0(1)\n"
                 "  ipm  %0\n"
                 "  srl  %0,28\n"
                 "0:"
                 : "=&d" (retval), "+m" (*p)
                 : "d" (oldval) , "d" (newval)
-                : "cc", "0");
+                : "cc", "0", "1" );
         return retval == 0;
 }
 
